@@ -20,9 +20,9 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <unistd.h>
-#include "flash.h"
 #include "programmer.h"
 #include "spi.h"
+#include "platform/udelay.h"
 
 /* Change this to #define if you want to test without a serial implementation */
 #undef FAKE_COMMUNICATION
