@@ -22,11 +22,11 @@
 #include <limits.h>
 #include <errno.h>
 #include <libusb.h>
-#include "flash.h"
 #include "chipdrivers.h"
 #include "programmer.h"
 #include "spi.h"
 #include "usbdev.h"
+#include "platform/udelay.h"
 
 /* LIBUSB_CALL ensures the right calling conventions on libusb callbacks.
  * However, the macro is not defined everywhere. m(
