@@ -35,6 +35,7 @@
 #include "hwaccess_physmap.h"
 #include "chipdrivers.h"
 #include "erasure_layout.h"
+#include "platform/udelay.h"
 
 const char flashrom_version[] = FLASHROM_VERSION;
 
@@ -1104,6 +1105,15 @@ static probe_func_t *lookup_probe_func_ptr(const struct flashchip *chip)
 	return NULL;
 }
 
+/*
+ * Probes the entries in flashchips array one by one, starting from `startchip` index.
+ * Probing keeps going until first match found or end of array reached.
+ *
+ * Returns:
+ * the position of the matched chip, i.e. index of the entry in flashchips array
+ * ERROR_FLASHROM_PROBE_NO_CHIPS_FOUND if no matches found
+ * ERROR_FLASHROM_PROBE_INTERNAL_ERROR if some unexpected error happened during this operation
+*/
 int probe_flash(struct registered_master *mst, int startchip, struct flashctx *flash, int force, const char *const chip_to_probe)
 {
 	const struct flashchip *chip;
@@ -1130,7 +1140,7 @@ int probe_flash(struct registered_master *mst, int startchip, struct flashctx *f
 		flash->chip = calloc(1, sizeof(*flash->chip));
 		if (!flash->chip) {
 			msg_gerr("Out of memory!\n");
-			return -1;
+			return ERROR_FLASHROM_PROBE_INTERNAL_ERROR;
 		}
 		*flash->chip = *chip;
 		flash->mst = mst;
@@ -1188,10 +1198,10 @@ notfound:
 	}
 
 	if (!flash->chip)
-		return -1;
+		return ERROR_FLASHROM_PROBE_NO_CHIPS_FOUND;
 
 	if (init_default_layout(flash) < 0)
-		return -1;
+		return ERROR_FLASHROM_PROBE_INTERNAL_ERROR;
 
 	tmp = flashbuses_to_text(flash->chip->bustype);
 	msg_cinfo("%s %s flash chip \"%s\" (%d kB, %s) ", force ? "Assuming" : "Found",
@@ -1221,7 +1231,7 @@ notfound:
 static void setup_progress_from_layout(struct flashctx *flashctx,
 				       enum flashrom_progress_stage stage)
 {
-	if (!flashctx->progress_callback)
+	if (!flashctx->progress_callback && !flashctx->deprecated_progress_callback)
 		return;
 
 	const struct flashrom_layout *const flash_layout = get_layout(flashctx);
@@ -1242,7 +1252,7 @@ static void setup_progress_from_layout_and_diff(struct flashctx *flashctx,
 						const void *want,
 						enum flashrom_progress_stage stage)
 {
-	if (!flashctx->progress_callback)
+	if (!flashctx->progress_callback && !flashctx->deprecated_progress_callback)
 		return;
 
 	const struct flashrom_layout *flash_layout = get_layout(flashctx);
@@ -1528,27 +1538,27 @@ static void nonfatal_help_message(void)
 	if (is_internal_programmer())
 		msg_gerr("This means we have to add special support for your board, programmer or flash\n"
 			 "chip. Please report this to the mailing list at flashrom@flashrom.org or on\n"
-			 "IRC (see https://www.flashrom.org/Contact for details), thanks!\n"
+			 "chat channels (see https://flashrom.org/contact.html for details), thanks!\n"
 			 "-------------------------------------------------------------------------------\n"
 			 "You may now reboot or simply leave the machine running.\n");
 	else
 		msg_gerr("Please check the connections (especially those to write protection pins) between\n"
 			 "the programmer and the flash chip. If you think the error is caused by flashrom\n"
-			 "please report this to the mailing list at flashrom@flashrom.org or on IRC (see\n"
-			 "https://www.flashrom.org/Contact for details), thanks!\n");
+			 "please report this to the mailing list at flashrom@flashrom.org or on chat (see\n"
+			 "https://flashrom.org/contact.html for details), thanks!\n");
 }
 
 void emergency_help_message(void)
 {
 	msg_gerr("Your flash chip is in an unknown state.\n");
 	if (is_internal_programmer())
-		msg_gerr("Get help on IRC (see https://www.flashrom.org/Contact) or mail\n"
+		msg_gerr("Get help on chat (see https://flashrom.org/contact.html) or mail\n"
 			"flashrom@flashrom.org with the subject \"FAILED: <your board name>\"!"
 			"-------------------------------------------------------------------------------\n"
 			"DO NOT REBOOT OR POWEROFF!\n");
 	else
 		msg_gerr("Please report this to the mailing list at flashrom@flashrom.org or\n"
-			 "on IRC (see https://www.flashrom.org/Contact for details), thanks!\n");
+			 "on chat (see https://flashrom.org/contact.html for details), thanks!\n");
 }
 
 void list_programmers_linebreak(int startcol, int cols, int paren)
