@@ -22,11 +22,11 @@
 #include <string.h>
 #include <stdbool.h>
 #include <stdlib.h>
-#include "flash.h"
 #include "programmer.h"
 #include "hwaccess_physmap.h"
 #include "spi.h"
 #include "ich_descriptors.h"
+#include "platform/udelay.h"
 
 /* Apollo Lake */
 #define APL_REG_FREG12		0xe0	/* 32 Bytes Flash Region 12 */
@@ -2237,7 +2237,7 @@ static int init_ich_default(const struct programmer_cfg *cfg, void *spibar, enum
 		if (ich_spi_rw_restricted)
 			msg_pinfo("Not all flash regions are freely accessible by flashrom. This is "
 				  "most likely\ndue to an active ME. Please see "
-				  "https://flashrom.org/ME for details.\n");
+				  "https://flashrom.org/user_docs/management_engine.html for details.\n");
 	}
 
 	/* Handle PR registers */
