@@ -23,7 +23,7 @@
 #include <stdbool.h>
 #include <stdint.h>
 
-#include "flash.h"	/* for chipaddr and flashctx */
+#include "spi.h"
 
 enum programmer_type {
 	PCI = 1, /* to detect uninitialized values */
@@ -97,6 +97,7 @@ extern const struct programmer_entry programmer_realtek_mst_i2c_spi;
 extern const struct programmer_entry programmer_satamv;
 extern const struct programmer_entry programmer_satasii;
 extern const struct programmer_entry programmer_serprog;
+extern const struct programmer_entry programmer_spidriver;
 extern const struct programmer_entry programmer_stlinkv3_spi;
 extern const struct programmer_entry programmer_usbblaster_spi;
 extern const struct programmer_entry programmer_dirtyjtag_spi;
@@ -216,10 +217,6 @@ extern const struct board_info boards_known[];
 extern const struct board_info laptops_known[];
 #endif
 
-/* udelay.c */
-void internal_sleep(unsigned int usecs);
-void default_delay(unsigned int usecs);
-
 #if CONFIG_INTERNAL == 1
 /* board_enable.c */
 int selfcheck_board_enables(void);
@@ -526,4 +523,6 @@ static inline bool spi_chip_4ba(const struct flashctx *const flash)
 		(flash->chip->feature_bits & (FEATURE_4BA_ENTER | FEATURE_4BA_ENTER_WREN | FEATURE_4BA_ENTER_EAR7));
 }
 
+enum chipbustype get_buses_supported(void);
+
 #endif				/* !__PROGRAMMER_H__ */
