@@ -129,7 +129,8 @@ static void report_nonwl_laptop_detected(const struct board_cfg *bcfg)
 	msg_perr("Laptops, notebooks and netbooks are difficult to support and we\n"
 		 "recommend to use the vendor flashing utility. The embedded controller\n"
 		 "(EC) in these machines often interacts badly with flashing.\n"
-		 "See the manpage and https://flashrom.org/Laptops for details.\n\n"
+		 "See the manpage and https://flashrom.org/contrib_howtos/laptops_and_ec.html\n"
+		 "for details.\n\n"
 		 "If flash is shared with the EC, erase is guaranteed to brick your laptop\n"
 		 "and write may brick your laptop.\n"
 		 "Read and probe may irritate your EC and cause fan failure, backlight\n"
