@@ -23,12 +23,12 @@
 #include <stdbool.h>
 #include <stdlib.h>
 #include <errno.h>
-#include "flash.h"
 #include "chipdrivers.h"
 #include "programmer.h"
 #include "hwaccess_physmap.h"
 #include "hwaccess_x86_io.h"
 #include "spi.h"
+#include "platform/udelay.h"
 
 #define ITE_SUPERIO_PORT1	0x2e
 #define ITE_SUPERIO_PORT2	0x4e
