@@ -12,16 +12,13 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
-#if defined(__i386__) || defined(__x86_64__)
-
 #include <stdlib.h>
 #include "flash.h"
+#include "programmer.h"
+#include "hwaccess_x86_io.h"
+#include "platform/pci.h"
 
 #define BIOS_ROM_ADDR		0x04
 #define BIOS_ROM_DATA		0x08
@@ -31,16 +28,19 @@
 
 #define PCI_VENDOR_ID_3COM	0x10b7
 
-uint32_t internal_conf;
-uint16_t id;
+struct nic3com_data {
+	uint32_t io_base_addr;
+	uint32_t internal_conf;
+	uint16_t id;
+};
 
-struct pcidev_status nics_3com[] = {
+static const struct dev_entry nics_3com[] = {
 	/* 3C90xB */
 	{0x10b7, 0x9055, OK, "3COM", "3C90xB: PCI 10/100 Mbps; shared 10BASE-T/100BASE-TX"},
 	{0x10b7, 0x9001, NT, "3COM", "3C90xB: PCI 10/100 Mbps; shared 10BASE-T/100BASE-T4" },
 	{0x10b7, 0x9004, OK, "3COM", "3C90xB: PCI 10BASE-T (TPO)" },
 	{0x10b7, 0x9005, NT, "3COM", "3C90xB: PCI 10BASE-T/10BASE2/AUI (COMBO)" },
-	{0x10b7, 0x9006, NT, "3COM", "3C90xB: PCI 10BASE-T/10BASE2 (TPC)" },
+	{0x10b7, 0x9006, OK, "3COM", "3C90xB: PCI 10BASE-T/10BASE2 (TPC)" },
 	{0x10b7, 0x900a, NT, "3COM", "3C90xB: PCI 10BASE-FL" },
 	{0x10b7, 0x905a, NT, "3COM", "3C90xB: PCI 10BASE-FX" },
 	{0x10b7, 0x9058, OK, "3COM", "3C905B: Cyclone 10/100/BNC" },
@@ -51,16 +51,69 @@ struct pcidev_status nics_3com[] = {
 	/* 3C980C */
 	{0x10b7, 0x9805, NT, "3COM", "3C980C: EtherLink Server 10/100 PCI (TX)" },
 
-	{},
+	{0},
 };
 
-int nic3com_init(void)
+static void nic3com_chip_writeb(const struct flashctx *flash, uint8_t val,
+				chipaddr addr)
 {
-	get_io_perms();
+	struct nic3com_data *data = flash->mst->par.data;
 
-	io_base_addr = pcidev_init(PCI_VENDOR_ID_3COM, PCI_BASE_ADDRESS_0,
-				   nics_3com, programmer_param);
-	id = pcidev_dev->device_id;
+	OUTL((uint32_t)addr, data->io_base_addr + BIOS_ROM_ADDR);
+	OUTB(val, data->io_base_addr + BIOS_ROM_DATA);
+}
+
+static uint8_t nic3com_chip_readb(const struct flashctx *flash,
+				  const chipaddr addr)
+{
+	struct nic3com_data *data = flash->mst->par.data;
+
+	OUTL((uint32_t)addr, data->io_base_addr + BIOS_ROM_ADDR);
+	return INB(data->io_base_addr + BIOS_ROM_DATA);
+}
+
+static int nic3com_shutdown(void *par_data)
+{
+	struct nic3com_data *data = par_data;
+	const uint16_t id = data->id;
+
+	/* 3COM 3C90xB cards need a special fixup. */
+	if (id == 0x9055 || id == 0x9001 || id == 0x9004 || id == 0x9005
+	    || id == 0x9006 || id == 0x900a || id == 0x905a || id == 0x9058) {
+		/* Select register window 3 and restore the receiver status. */
+		OUTW(SELECT_REG_WINDOW + 3, data->io_base_addr + INT_STATUS);
+		OUTL(data->internal_conf, data->io_base_addr + INTERNAL_CONFIG);
+	}
+
+	free(data);
+	return 0;
+}
+
+static const struct par_master par_master_nic3com = {
+	.chip_readb	= nic3com_chip_readb,
+	.chip_writeb	= nic3com_chip_writeb,
+	.shutdown	= nic3com_shutdown,
+};
+
+static int nic3com_init(const struct programmer_cfg *cfg)
+{
+	struct pci_dev *dev = NULL;
+	uint32_t io_base_addr = 0;
+	uint32_t internal_conf = 0;
+	uint16_t id;
+
+	if (rget_io_perms())
+		return 1;
+
+	dev = pcidev_init(cfg, nics_3com, PCI_BASE_ADDRESS_0);
+	if (!dev)
+		return 1;
+
+	io_base_addr = pcidev_readbar(dev, PCI_BASE_ADDRESS_0);
+	if (!io_base_addr)
+		return 1;
+
+	id = dev->device_id;
 
 	/* 3COM 3C90xB cards need a special fixup. */
 	if (id == 0x9055 || id == 0x9001 || id == 0x9004 || id == 0x9005
@@ -80,13 +133,20 @@ int nic3com_init(void)
 	 */
 	OUTW(SELECT_REG_WINDOW + 0, io_base_addr + INT_STATUS);
 
-	buses_supported = CHIP_BUSTYPE_PARALLEL;
+	struct nic3com_data *data = calloc(1, sizeof(*data));
+	if (!data) {
+		msg_perr("Unable to allocate space for PAR master data\n");
+		goto init_err_cleanup_exit;
+	}
+	data->io_base_addr = io_base_addr;
+	data->internal_conf = internal_conf;
+	data->id = id;
 
-	return 0;
-}
+	max_rom_decode.parallel = 128 * 1024;
 
-int nic3com_shutdown(void)
-{
+	return register_par_master(&par_master_nic3com, BUS_PARALLEL, data);
+
+init_err_cleanup_exit:
 	/* 3COM 3C90xB cards need a special fixup. */
 	if (id == 0x9055 || id == 0x9001 || id == 0x9004 || id == 0x9005
 	    || id == 0x9006 || id == 0x900a || id == 0x905a || id == 0x9058) {
@@ -94,25 +154,12 @@ int nic3com_shutdown(void)
 		OUTW(SELECT_REG_WINDOW + 3, io_base_addr + INT_STATUS);
 		OUTL(internal_conf, io_base_addr + INTERNAL_CONFIG);
 	}
-
-	free(programmer_param);
-	pci_cleanup(pacc);
-	release_io_perms();
-	return 0;
+	return 1;
 }
 
-void nic3com_chip_writeb(uint8_t val, chipaddr addr)
-{
-	OUTL((uint32_t)addr, io_base_addr + BIOS_ROM_ADDR);
-	OUTB(val, io_base_addr + BIOS_ROM_DATA);
-}
-
-uint8_t nic3com_chip_readb(const chipaddr addr)
-{
-	OUTL((uint32_t)addr, io_base_addr + BIOS_ROM_ADDR);
-	return INB(io_base_addr + BIOS_ROM_DATA);
-}
-
-#else
-#error PCI port I/O access is not supported on this architecture yet.
-#endif
+const struct programmer_entry programmer_nic3com = {
+	.name			= "nic3com",
+	.type			= PCI,
+	.devs.dev		= nics_3com,
+	.init			= nic3com_init,
+};
