@@ -31,11 +31,11 @@
 
 #include <stdlib.h>
 #include <unistd.h>
-#include "flash.h"
 #include "spi.h"
 #include "programmer.h"
 #include "hwaccess_physmap.h"
 #include "platform/pci.h"
+#include "platform/udelay.h"
 
 #define PCI_VENDOR_ID_INTEL 0x8086
 #define MEMMAP_SIZE 0x1c /* Only EEC, EERD and EEWR are needed. */
