@@ -39,7 +39,60 @@ void dummy_probe_lifecycle_test_success(void **state)
 		.fallback_open_state = &dummy_fallback_open_state,
 	};
 
-	run_probe_lifecycle(state, &dummy_io, &programmer_dummy, "bus=spi,emulate=W25Q128FV", "W25Q128.V");
+	const char *expected_matched_names[1] = {"W25Q128.V"};
+	run_probe_v2_lifecycle(state, &dummy_io, &programmer_dummy, "bus=spi,emulate=W25Q128FV", "W25Q128.V",
+				expected_matched_names, 1);
+}
+
+void dummy_probe_v2_one_match_for_W25Q128FV(void **state)
+{
+	struct io_mock_fallback_open_state dummy_fallback_open_state = {
+		.noc = 0,
+		.paths = { NULL },
+	};
+	const struct io_mock dummy_io = {
+		.fallback_open_state = &dummy_fallback_open_state,
+	};
+
+	const char *expected_matched_names[1] = {"W25Q128.V"};
+	run_probe_v2_lifecycle(state, &dummy_io, &programmer_dummy, "bus=spi,emulate=W25Q128FV",
+				NULL, /* any chip name */
+				expected_matched_names, 1);
+}
+
+void dummy_probe_v2_six_matches_for_MX25L6436(void **state)
+{
+	struct io_mock_fallback_open_state dummy_fallback_open_state = {
+		.noc = 0,
+		.paths = { NULL },
+	};
+	const struct io_mock dummy_io = {
+		.fallback_open_state = &dummy_fallback_open_state,
+	};
+
+	const char *expected_matched_names[6] = {"MX25L6405",
+						 "MX25L6405D",
+						 "MX25L6406E/MX25L6408E",
+						 "MX25L6436E/MX25L6445E/MX25L6465E",
+						 "MX25L6473E",
+						 "MX25L6473F"};
+	run_probe_v2_lifecycle(state, &dummy_io, &programmer_dummy, "bus=spi,emulate=MX25L6436",
+				NULL, /* any chip name */
+				expected_matched_names, 6);
+}
+
+void dummy_probe_v2_no_matches_found(void **state)
+{
+	struct io_mock_fallback_open_state dummy_fallback_open_state = {
+		.noc = 0,
+		.paths = { NULL },
+	};
+	const struct io_mock dummy_io = {
+		.fallback_open_state = &dummy_fallback_open_state,
+	};
+
+	run_probe_v2_lifecycle(state, &dummy_io, &programmer_dummy, "bus=spi,emulate=MX25L6436",
+				"NONEXISTENT", NULL /* no matched names */, 0);
 }
 
 void dummy_probe_variable_size_test_success(void **state)
@@ -52,7 +105,9 @@ void dummy_probe_variable_size_test_success(void **state)
 		.fallback_open_state = &dummy_fallback_open_state,
 	};
 
-	run_probe_lifecycle(state, &dummy_io, &programmer_dummy, "size=8388608,emulate=VARIABLE_SIZE", "Opaque flash chip");
+	const char *expected_matched_names[1] = {"Opaque flash chip"};
+	run_probe_v2_lifecycle(state, &dummy_io, &programmer_dummy, "size=8388608,emulate=VARIABLE_SIZE", "Opaque flash chip",
+				expected_matched_names, 1);
 }
 
 void dummy_init_fails_unhandled_param_test_success(void **state)
@@ -163,6 +218,9 @@ void dummy_freq_param_init(void **state)
 #else
 	SKIP_TEST(dummy_basic_lifecycle_test_success)
 	SKIP_TEST(dummy_probe_lifecycle_test_success)
+	SKIP_TEST(dummy_probe_v2_one_match_for_W25Q128FV)
+	SKIP_TEST(dummy_probe_v2_six_matches_for_MX25L6436)
+	SKIP_TEST(dummy_probe_v2_no_matches_found)
 	SKIP_TEST(dummy_probe_variable_size_test_success)
 	SKIP_TEST(dummy_init_fails_unhandled_param_test_success)
 	SKIP_TEST(dummy_init_success_invalid_param_test_success)
