@@ -13,6 +13,7 @@
 
 #include <include/test.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "tests.h"
@@ -51,12 +52,16 @@ struct test_case {
 	uint8_t written_protected_buf[MOCK_CHIP_SIZE]; /* Expected content after write with protected region. */
 	struct erase_invoke eraseblocks_expected[MOCK_CHIP_SIZE]; /* Expected order of eraseblocks invocations. */
 	unsigned int eraseblocks_expected_ind; /* Expected number of eraseblocks invocations. */
+	struct erase_invoke write_eraseblocks_expected[MOCK_CHIP_SIZE]; /* Expected order of eraseblocks invocations. */
+	unsigned int write_eraseblocks_expected_ind; /* Expected number of eraseblocks invocations. */
 	char erase_test_name[40]; /* Test case display name for testing erase operation. */
 	char write_test_name[40]; /* Test case display name for testing write operation. */
 };
 
 struct all_state {
 	uint8_t buf[MIN_REAL_CHIP_SIZE]; /* Buffer emulating the memory of the mock chip. */
+	bool was_modified[MIN_REAL_CHIP_SIZE]; /* Which bytes were modified, 0x1 if byte was modified. */
+	bool was_verified[MIN_REAL_CHIP_SIZE]; /* Which bytes were verified, 0x1 if byte was verified. */
 	struct erase_invoke eraseblocks_actual[MOCK_CHIP_SIZE]; /* The actual order of eraseblocks invocations. */
 	unsigned int eraseblocks_actual_ind; /* Actual number of eraseblocks invocations. */
 	const struct test_case* current_test_case; /* Currently executed test case. */
@@ -64,29 +69,46 @@ struct all_state {
 
 static int read_chip(struct flashctx *flash, uint8_t *buf, unsigned int start, unsigned int len)
 {
-	if (start + len <= MOCK_CHIP_SIZE)
+	if (start < MOCK_CHIP_SIZE)
 		LOG_READ_WRITE_FUNC;
 
 	assert_in_range(start + len, 0, MIN_REAL_CHIP_SIZE);
 
 	memcpy(buf, &g_state.buf[start], len);
+
+	/* If these bytes were modified before => current read op is verify op, track it */
+	bool bytes_modified = false;
+	for (unsigned int i = start; i < start + len; i++)
+		if (g_state.was_modified[i]) {
+			bytes_modified = true;
+			break;
+		}
+	if (bytes_modified)
+		memset(&g_state.was_verified[start], true, len);
+
 	return 0;
 }
 
 static int write_chip(struct flashctx *flash, const uint8_t *buf, unsigned int start, unsigned int len)
 {
-	if (start + len <= MOCK_CHIP_SIZE)
+	if (start < MOCK_CHIP_SIZE)
 		LOG_READ_WRITE_FUNC;
 
 	assert_in_range(start + len, 0, MIN_REAL_CHIP_SIZE);
 
 	memcpy(&g_state.buf[start], buf, len);
+
+	/* Track the bytes were written */
+	memset(&g_state.was_modified[start], true, len);
+	/* Clear the records of previous verification, if there were any */
+	memset(&g_state.was_verified[start], false, len);
+
 	return 0;
 }
 
 static int block_erase_chip_tagged(struct flashctx *flash, enum block_erase_func erase_func, unsigned int blockaddr, unsigned int blocklen)
 {
-	if (blockaddr + blocklen <= MOCK_CHIP_SIZE) {
+	if (blockaddr < MOCK_CHIP_SIZE) {
 		LOG_ERASE_FUNC;
 
 		/* Register eraseblock invocation. */
@@ -101,6 +123,12 @@ static int block_erase_chip_tagged(struct flashctx *flash, enum block_erase_func
 	assert_in_range(blockaddr + blocklen, 0, MIN_REAL_CHIP_SIZE);
 
 	memset(&g_state.buf[blockaddr], ERASE_VALUE, blocklen);
+
+	/* Track the bytes were erased */
+	memset(&g_state.was_modified[blockaddr], true, blocklen);
+	/* Clear the records of previous verification, if there were any */
+	memset(&g_state.was_verified[blockaddr], false, blocklen);
+
 	return 0;
 }
 
@@ -186,6 +214,29 @@ static struct flashchip chip_8_16 = {
 	},
 };
 
+static struct flashchip chip_1_4_16 = {
+	.vendor		= "aklm",
+	/* See comment on previous chip. */
+	.total_size	= 1,
+	.tested		= TEST_OK_PREW,
+	.gran		= WRITE_GRAN_1BYTE,
+	.read		= TEST_READ_INJECTOR,
+	.write		= TEST_WRITE_INJECTOR,
+	.block_erasers	=
+	{
+		{
+			.eraseblocks = { {1, MIN_REAL_CHIP_SIZE} },
+			.block_erase = TEST_ERASE_INJECTOR_1,
+		}, {
+			.eraseblocks = { {4, MIN_REAL_CHIP_SIZE / 4} },
+			.block_erase = TEST_ERASE_INJECTOR_3,
+		}, {
+			.eraseblocks = { {16, MIN_REAL_CHIP_SIZE / 16} },
+			.block_erase = TEST_ERASE_INJECTOR_5,
+		}
+	},
+};
+
 #define BLOCK_ERASE_FUNC(n) \
 	static int block_erase_chip_ ## n (struct flashctx *flash, unsigned int blockaddr, unsigned int blocklen) { \
 		return block_erase_chip_tagged(flash, TEST_ERASE_INJECTOR_ ## n, blockaddr, blocklen); \
@@ -196,9 +247,15 @@ BLOCK_ERASE_FUNC(3)
 BLOCK_ERASE_FUNC(4)
 BLOCK_ERASE_FUNC(5)
 
-static void setup_chip(struct flashrom_flashctx *flashctx, struct flashrom_layout **layout,
+/*
+ * Returns the offset how far we need to verify mock chip memory.
+ * Which is minimum out of MOCK_CHIP_SIZE and the end of the logical layout.
+ */
+static chipoff_t setup_chip(struct flashrom_flashctx *flashctx, struct flashrom_layout **layout,
 			const char *programmer_param, struct test_case *current_test_case)
 {
+	chipoff_t verify_end_boundary = MOCK_CHIP_SIZE - 1;
+
 	g_test_write_injector = write_chip;
 	g_test_read_injector = read_chip;
 	/* Each erasefunc corresponds to an operation that erases a block of
@@ -223,6 +280,14 @@ static void setup_chip(struct flashrom_flashctx *flashctx, struct flashrom_layou
 	memset(g_state.eraseblocks_actual, 0, MOCK_CHIP_SIZE * sizeof(struct erase_invoke));
 	g_state.eraseblocks_actual_ind = 0;
 
+	/* Clear the tracking of each byte modified. */
+	memset(g_state.was_modified, false, MIN_REAL_CHIP_SIZE);
+	/* Clear the tracking of each byte verified. */
+	memset(g_state.was_verified, false, MIN_REAL_CHIP_SIZE);
+
+	/* Set the flag to verify after writing on chip */
+	flashrom_flag_set(flashctx, FLASHROM_FLAG_VERIFY_AFTER_WRITE, true);
+
 	flashctx->chip = current_test_case->chip;
 
 	printf("Creating layout ... ");
@@ -236,6 +301,12 @@ static void setup_chip(struct flashrom_flashctx *flashctx, struct flashrom_layou
 					current_test_case->regions[i].end,
 					current_test_case->regions[i].name));
 		assert_int_equal(0, flashrom_layout_include_region(*layout, current_test_case->regions[i].name));
+
+		if (current_test_case->regions[i].end < MOCK_CHIP_SIZE - 1)
+			verify_end_boundary = current_test_case->regions[i].end;
+		else
+			verify_end_boundary = MOCK_CHIP_SIZE - 1;
+
 		i++;
 	}
 
@@ -252,6 +323,8 @@ static void setup_chip(struct flashrom_flashctx *flashctx, struct flashrom_layou
 	/* Assignment below normally happens while probing, but this test is not probing. */
 	flashctx->mst = &registered_masters[0];
 	printf("done\n");
+
+	return verify_end_boundary;
 }
 
 static void teardown_chip(struct flashrom_layout **layout)
@@ -293,6 +366,8 @@ static struct test_case test_cases[] = {
 				 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
+		.write_eraseblocks_expected_ind = 1,
 		.erase_test_name = "Erase test case #0",
 		.write_test_name = "Write test case #0",
 	}, {
@@ -306,7 +381,7 @@ static struct test_case test_cases[] = {
 		.chip =		&chip_1_2_4_8_16,
 		.regions =	{{0, MOCK_CHIP_SIZE/2 - 1, "part1"}, {MOCK_CHIP_SIZE/2, MIN_REAL_CHIP_SIZE - 1, "part2"}},
 		.initial_buf =	{0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,
-				 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff},
+				 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xf1},
 		.erased_buf =	{ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE,
 					ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE,
 					ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE,
@@ -315,6 +390,10 @@ static struct test_case test_cases[] = {
 				 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f},
 		.eraseblocks_expected = {{0x8, 0x8, TEST_ERASE_INJECTOR_4}, {0x0, 0x8, TEST_ERASE_INJECTOR_4}},
 		.eraseblocks_expected_ind = 2,
+		.write_eraseblocks_expected = {{0x9, 0x1, TEST_ERASE_INJECTOR_1},
+						{0xa, 0x2, TEST_ERASE_INJECTOR_2},
+						{0xc, 0x4, TEST_ERASE_INJECTOR_3}},
+		.write_eraseblocks_expected_ind = 3,
 		.erase_test_name = "Erase test case #1",
 		.write_test_name = "Write test case #1",
 	}, {
@@ -343,6 +422,11 @@ static struct test_case test_cases[] = {
 			{0x0, 0x8, TEST_ERASE_INJECTOR_4}
 		},
 		.eraseblocks_expected_ind = 5,
+		.write_eraseblocks_expected = {
+			{0x6, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x8, 0x1, TEST_ERASE_INJECTOR_1}
+		},
+		.write_eraseblocks_expected_ind = 2,
 		.erase_test_name = "Erase test case #2",
 		.write_test_name = "Write test case #2",
 	}, {
@@ -365,6 +449,8 @@ static struct test_case test_cases[] = {
 				 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x4, 0x4, TEST_ERASE_INJECTOR_3}},
+		.write_eraseblocks_expected_ind = 1,
 		.erase_test_name = "Erase test case #3",
 		.write_test_name = "Write test case #3",
 	}, {
@@ -387,6 +473,8 @@ static struct test_case test_cases[] = {
 				 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x0, 0x4, TEST_ERASE_INJECTOR_3}, {0x8, 0x8, TEST_ERASE_INJECTOR_4}},
+		.write_eraseblocks_expected_ind = 2,
 		.erase_test_name = "Erase test case #4",
 		.write_test_name = "Write test case #4",
 	}, {
@@ -409,6 +497,11 @@ static struct test_case test_cases[] = {
 				 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x6, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x4, 0x2, TEST_ERASE_INJECTOR_2},
+						{0x0, 0x4, TEST_ERASE_INJECTOR_3},
+						{0x8, 0x8, TEST_ERASE_INJECTOR_4}},
+		.write_eraseblocks_expected_ind = 4,
 		.erase_test_name = "Erase test case #5",
 		.write_test_name = "Write test case #5",
 	}, {
@@ -431,6 +524,8 @@ static struct test_case test_cases[] = {
 				 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x7, 0x1, TEST_ERASE_INJECTOR_1}},
+		.write_eraseblocks_expected_ind = 1,
 		.erase_test_name = "Erase test case #6",
 		.write_test_name = "Write test case #6",
 	}, {
@@ -462,6 +557,17 @@ static struct test_case test_cases[] = {
 			{0x0, 0x2, TEST_ERASE_INJECTOR_2}
 		},
 		.eraseblocks_expected_ind = 8,
+		.write_eraseblocks_expected = {
+			{0xf, 0x1, TEST_ERASE_INJECTOR_1},
+			{0xe, 0x1, TEST_ERASE_INJECTOR_1},
+			{0xc, 0x2, TEST_ERASE_INJECTOR_2},
+			{0x8, 0x4, TEST_ERASE_INJECTOR_3},
+			{0x3, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x4, 0x4, TEST_ERASE_INJECTOR_3},
+			{0x2, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x0, 0x2, TEST_ERASE_INJECTOR_2}
+		},
+		.write_eraseblocks_expected_ind = 8,
 		.erase_test_name = "Erase test case #7",
 		.write_test_name = "Write test case #7",
 	}, {
@@ -484,6 +590,8 @@ static struct test_case test_cases[] = {
 				 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
+		.write_eraseblocks_expected_ind = 1,
 		.erase_test_name = "Erase test case #8",
 		.write_test_name = "Write test case #8",
 	}, {
@@ -498,7 +606,7 @@ static struct test_case test_cases[] = {
 		.regions =	{{0, MOCK_CHIP_SIZE/2 - 1, "part1"}, {MOCK_CHIP_SIZE/2, MOCK_CHIP_SIZE - 1, "part2"},
 					{MOCK_CHIP_SIZE, MIN_REAL_CHIP_SIZE - 1, "longtail"}},
 		.initial_buf =	{0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,
-				 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff},
+				 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xf1},
 		.erased_buf =	{ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE,
 					ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE,
 					ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE,
@@ -507,6 +615,14 @@ static struct test_case test_cases[] = {
 				 0xf8, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f},
 		.eraseblocks_expected = {{0x8, 0x8, TEST_ERASE_INJECTOR_4}, {0x0, 0x8, TEST_ERASE_INJECTOR_4}},
 		.eraseblocks_expected_ind = 2,
+		.write_eraseblocks_expected = {{0x9, 0x1, TEST_ERASE_INJECTOR_1},
+						{0xa, 0x1, TEST_ERASE_INJECTOR_1},
+						{0xb, 0x1, TEST_ERASE_INJECTOR_1},
+						{0xc, 0x1, TEST_ERASE_INJECTOR_1},
+						{0xd, 0x1, TEST_ERASE_INJECTOR_1},
+						{0xe, 0x1, TEST_ERASE_INJECTOR_1},
+						{0xf, 0x1, TEST_ERASE_INJECTOR_1}},
+		.write_eraseblocks_expected_ind = 7,
 		.erase_test_name = "Erase test case #9",
 		.write_test_name = "Write test case #9",
 	}, {
@@ -539,6 +655,11 @@ static struct test_case test_cases[] = {
 			{0x0, 0x8, TEST_ERASE_INJECTOR_4}
 		},
 		.eraseblocks_expected_ind = 9,
+		.write_eraseblocks_expected = {
+			{0x6, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x8, 0x1, TEST_ERASE_INJECTOR_1}
+		},
+		.write_eraseblocks_expected_ind = 2,
 		.erase_test_name = "Erase test case #10",
 		.write_test_name = "Write test case #10",
 	}, {
@@ -561,6 +682,11 @@ static struct test_case test_cases[] = {
 				 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x4, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x5, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x6, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x7, 0x1, TEST_ERASE_INJECTOR_1}},
+		.write_eraseblocks_expected_ind = 4,
 		.erase_test_name = "Erase test case #11",
 		.write_test_name = "Write test case #11",
 	}, {
@@ -583,6 +709,12 @@ static struct test_case test_cases[] = {
 				 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x0, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x1, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x2, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x3, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x8, 0x8, TEST_ERASE_INJECTOR_4}},
+		.write_eraseblocks_expected_ind = 5,
 		.erase_test_name = "Erase test case #12",
 		.write_test_name = "Write test case #12",
 	}, {
@@ -605,6 +737,15 @@ static struct test_case test_cases[] = {
 				 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x0, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x1, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x2, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x3, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x4, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x5, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x6, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x8, 0x8, TEST_ERASE_INJECTOR_4}},
+		.write_eraseblocks_expected_ind = 8,
 		.erase_test_name = "Erase test case #13",
 		.write_test_name = "Write test case #13",
 	}, {
@@ -627,6 +768,8 @@ static struct test_case test_cases[] = {
 				 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x7, 0x1, TEST_ERASE_INJECTOR_1}},
+		.write_eraseblocks_expected_ind = 1,
 		.erase_test_name = "Erase test case #14",
 		.write_test_name = "Write test case #14",
 	}, {
@@ -667,6 +810,25 @@ static struct test_case test_cases[] = {
 			{0x2, 0x1, TEST_ERASE_INJECTOR_1},
 		},
 		.eraseblocks_expected_ind = 16,
+		.write_eraseblocks_expected = {
+			{0xf, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x8, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x9, 0x1, TEST_ERASE_INJECTOR_1},
+			{0xa, 0x1, TEST_ERASE_INJECTOR_1},
+			{0xb, 0x1, TEST_ERASE_INJECTOR_1},
+			{0xc, 0x1, TEST_ERASE_INJECTOR_1},
+			{0xd, 0x1, TEST_ERASE_INJECTOR_1},
+			{0xe, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x3, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x4, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x5, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x6, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x7, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x0, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x1, 0x1, TEST_ERASE_INJECTOR_1},
+			{0x2, 0x1, TEST_ERASE_INJECTOR_1},
+		},
+		.write_eraseblocks_expected_ind = 16,
 		.erase_test_name = "Erase test case #15",
 		.write_test_name = "Write test case #15",
 	}, {
@@ -695,6 +857,13 @@ static struct test_case test_cases[] = {
 			{0x0, 0x8, TEST_ERASE_INJECTOR_4},
 		},
 		.eraseblocks_expected_ind = 4,
+		.write_eraseblocks_expected = {
+			{0x8, 0x8, TEST_ERASE_INJECTOR_4},
+			{0x0, 0x10, TEST_ERASE_INJECTOR_5},
+			{0x0, 0x8, TEST_ERASE_INJECTOR_4},
+			{0x0, 0x8, TEST_ERASE_INJECTOR_4},
+		},
+		.write_eraseblocks_expected_ind = 4,
 		.erase_test_name = "Erase test case #16",
 		.write_test_name = "Write test case #16",
 	}, {
@@ -717,6 +886,8 @@ static struct test_case test_cases[] = {
 				 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16},
 		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}, {0x0, 0x8, TEST_ERASE_INJECTOR_4}},
 		.eraseblocks_expected_ind = 2,
+		.write_eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}, {0x0, 0x8, TEST_ERASE_INJECTOR_4}},
+		.write_eraseblocks_expected_ind = 2,
 		.erase_test_name = "Erase test case #17",
 		.write_test_name = "Write test case #17",
 	}, {
@@ -739,6 +910,8 @@ static struct test_case test_cases[] = {
 				 0x14, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16},
 		.eraseblocks_expected = {{0x8, 0x8, TEST_ERASE_INJECTOR_4}, {0x0, 0x10, TEST_ERASE_INJECTOR_5}},
 		.eraseblocks_expected_ind = 2,
+		.write_eraseblocks_expected = {{0x8, 0x8, TEST_ERASE_INJECTOR_4}, {0x0, 0x10, TEST_ERASE_INJECTOR_5}},
+		.write_eraseblocks_expected_ind = 2,
 		.erase_test_name = "Erase test case #18",
 		.write_test_name = "Write test case #18",
 	}, {
@@ -760,8 +933,43 @@ static struct test_case test_cases[] = {
 				 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0},
 		.eraseblocks_expected = {{0x0, 0x8, TEST_ERASE_INJECTOR_4}},
 		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x0, 0x8, TEST_ERASE_INJECTOR_4}},
+		.write_eraseblocks_expected_ind = 1,
 		.erase_test_name = "Erase test case #19",
 		.write_test_name = "Write test case #19",
+	}, {
+		/*
+		 * Test case #20
+		 *
+		 * Initial vs written: 3s+1d+3s+1d+3s+1d+3d+1s
+		 * Layout with one region covering the whole chip
+		 * Chip with eraseblocks 1, 4, 16.
+		 */
+		.chip =		&chip_1_4_16,
+		.regions =	{{0, MOCK_CHIP_SIZE - 1, "mock chip"}, {MOCK_CHIP_SIZE, MIN_REAL_CHIP_SIZE - 1, "longtail"}},
+		.initial_buf =	{0x0, 0x1, 0x2, 0xf, 0x4, 0x5, 0x6, 0xf,
+				 0x8, 0x9, 0xa, 0xf, 0xf, 0xf, 0xf, 0x1f},
+		.erased_buf =	{ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE,
+					ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE,
+					ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE,
+					ERASE_VALUE, ERASE_VALUE, ERASE_VALUE, ERASE_VALUE},
+		.written_buf =	{0x01, 0x11, 0x22, 0xf, 0x44, 0x55, 0x66, 0xf,
+				 0x88, 0x99, 0xaa, 0xf, 0xf, 0xf, 0xf, 0xf1},
+		.eraseblocks_expected = {{0x0, 0x10, TEST_ERASE_INJECTOR_5}},
+		.eraseblocks_expected_ind = 1,
+		.write_eraseblocks_expected = {{0x0, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x1, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x2, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x4, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x5, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x6, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x8, 0x1, TEST_ERASE_INJECTOR_1},
+						{0x9, 0x1, TEST_ERASE_INJECTOR_1},
+						{0xa, 0x1, TEST_ERASE_INJECTOR_1},
+						{0xf, 0x1, TEST_ERASE_INJECTOR_1}},
+		.write_eraseblocks_expected_ind = 10,
+		.erase_test_name = "Erase test case #20",
+		.write_test_name = "Write test case #20",
 	},
 };
 
@@ -771,7 +979,7 @@ static struct test_case test_cases[] = {
 
 /*
  * Setup all test cases with protected region.
- * Protected region is the same for all test cases, between bytes 8 - 15.
+ * Protected region is the same for all test cases, between bytes START_PROTECTED_REGION and up to END_PROTECTED_REGION.
  */
 static struct test_case test_cases_protected_region[] = {
 	{
@@ -983,6 +1191,7 @@ struct CMUnitTest *get_erase_func_algo_tests(size_t *num_tests) {
 }
 
 static void test_erase_fails_for_unwritable_region(void **);
+static void test_write_with_sacrifice_ratio50(void **);
 static void erase_unwritable_regions_skipflag_on_test_success(void **);
 static void write_unwritable_regions_skipflag_on_test_success(void **);
 
@@ -992,7 +1201,7 @@ static void write_unwritable_regions_skipflag_on_test_success(void **);
  */
 struct CMUnitTest *get_erase_protected_region_algo_tests(size_t *num_tests) {
 	const size_t num_parameterized = ARRAY_SIZE(test_cases_protected_region);
-	const size_t num_unparameterized = 1;
+	const size_t num_unparameterized = 2;
 	// Twice the number of parameterized test cases, because each test case is run twice:
 	// for erase and write.
 	const size_t num_cases = num_parameterized * 2 + num_unparameterized;
@@ -1023,7 +1232,11 @@ struct CMUnitTest *get_erase_protected_region_algo_tests(size_t *num_tests) {
 			(const struct CMUnitTest) {
 				.name = "erase failure for unskipped unwritable regions",
 				.test_func = test_erase_fails_for_unwritable_region,
-			}
+			},
+			(const struct CMUnitTest) {
+				.name = "write with sacrifice ratio 50",
+				.test_func = test_write_with_sacrifice_ratio50,
+			},
 		},
 		sizeof(*all_cases) * num_unparameterized
 	);
@@ -1045,7 +1258,7 @@ void erase_function_algo_test_success(void **state)
 
 	struct flashrom_layout *layout;
 
-	setup_chip(&flashctx, &layout, param, current_test_case);
+	const chipoff_t verify_end_boundary = setup_chip(&flashctx, &layout, param, current_test_case);
 
 	printf("%s started.\n", current_test_case->erase_test_name);
 	int ret = flashrom_flash_erase(&flashctx);
@@ -1060,6 +1273,13 @@ void erase_function_algo_test_success(void **state)
 	int eraseblocks_invocations = (g_state.eraseblocks_actual_ind ==
 					current_test_case->eraseblocks_expected_ind);
 
+	int chip_verified = 1;
+	for (unsigned int i = 0; i <= verify_end_boundary; i++)
+		if (g_state.was_modified[i] && !g_state.was_verified[i]) {
+			chip_verified = 0; /* byte was modified, but not verified after */
+			printf("Error: byte 0x%x, modified: %d, verified: %d\n", i, g_state.was_modified[i], g_state.was_verified[i]);
+		}
+
 	if (chip_erased)
 		printf("Erased chip memory state for %s is CORRECT\n",
 			current_test_case->erase_test_name);
@@ -1083,10 +1303,18 @@ void erase_function_algo_test_success(void **state)
 			current_test_case->eraseblocks_expected_ind,
 			g_state.eraseblocks_actual_ind);
 
+	if (chip_verified)
+		printf("Erased chip memory state for %s was verified successfully\n",
+			current_test_case->erase_test_name);
+	else
+		printf("Erased chip memory state for %s was NOT verified completely\n",
+			current_test_case->erase_test_name);
+
 	all_erase_tests_result |= ret;
 	all_erase_tests_result |= !chip_erased;
 	all_erase_tests_result |= !eraseblocks_in_order;
 	all_erase_tests_result |= !eraseblocks_invocations;
+	all_erase_tests_result |= !chip_verified;
 
 	teardown_chip(&layout);
 
@@ -1108,7 +1336,7 @@ void write_function_algo_test_success(void **state)
 
 	struct flashrom_layout *layout;
 
-	setup_chip(&flashctx, &layout, param, current_test_case);
+	const chipoff_t verify_end_boundary = setup_chip(&flashctx, &layout, param, current_test_case);
 	memcpy(&newcontents, current_test_case->written_buf, MOCK_CHIP_SIZE);
 
 	printf("%s started.\n", current_test_case->write_test_name);
@@ -1117,6 +1345,20 @@ void write_function_algo_test_success(void **state)
 
 	int chip_written = !memcmp(g_state.buf, current_test_case->written_buf, MOCK_CHIP_SIZE);
 
+	int eraseblocks_in_order = !memcmp(g_state.eraseblocks_actual,
+					current_test_case->write_eraseblocks_expected,
+					current_test_case->write_eraseblocks_expected_ind * sizeof(struct erase_invoke));
+
+	int eraseblocks_invocations = (g_state.eraseblocks_actual_ind ==
+					current_test_case->write_eraseblocks_expected_ind);
+
+	int chip_verified = 1;
+	for (unsigned int i = 0; i <= verify_end_boundary; i++)
+		if (g_state.was_modified[i] && !g_state.was_verified[i]) {
+			chip_verified = 0; /* the byte was modified, but not verified after */
+			printf("Error: byte 0x%x, modified: %d, verified: %d\n", i, g_state.was_modified[i], g_state.was_verified[i]);
+		}
+
 	if (chip_written)
 		printf("Written chip memory state for %s is CORRECT\n",
 			current_test_case->write_test_name);
@@ -1124,8 +1366,34 @@ void write_function_algo_test_success(void **state)
 		printf("Written chip memory state for %s is WRONG\n",
 			current_test_case->write_test_name);
 
+	if (eraseblocks_in_order)
+		printf("Eraseblocks order of invocation for %s is CORRECT\n",
+			current_test_case->write_test_name);
+	else
+		printf("Eraseblocks order of invocation for %s is WRONG\n",
+			current_test_case->write_test_name);
+
+	if (eraseblocks_invocations)
+		printf("Eraseblocks number of invocations for %s is CORRECT\n",
+			current_test_case->write_test_name);
+	else
+		printf("Eraseblocks number of invocations for %s is WRONG, expected %d actual %d\n",
+			current_test_case->write_test_name,
+			current_test_case->write_eraseblocks_expected_ind,
+			g_state.eraseblocks_actual_ind);
+
+	if (chip_verified)
+		printf("Written chip memory state for %s was verified successfully\n",
+			current_test_case->write_test_name);
+	else
+		printf("Written chip memory state for %s was NOT verified completely\n",
+			current_test_case->write_test_name);
+
 	all_write_test_result |= ret;
 	all_write_test_result |= !chip_written;
+	all_write_test_result |= !eraseblocks_in_order;
+	all_write_test_result |= !eraseblocks_invocations;
+	all_write_test_result |= !chip_verified;
 
 	teardown_chip(&layout);
 
@@ -1147,7 +1415,7 @@ static void get_protected_region(const struct flashctx *flash, unsigned int addr
 		region->name		= strdup("protected");
 		region->start		= START_PROTECTED_REGION;
 		region->end		= END_PROTECTED_REGION;
-		region->read_prot	= true;
+		region->read_prot	= false;
 		region->write_prot	= true;
 	} else {
 		region->name		= strdup("tail");
@@ -1197,6 +1465,12 @@ static int block_erase_chip_with_protected_region(struct flashctx *flash, enum b
 	}
 
 	memset(&g_state.buf[blockaddr], ERASE_VALUE, blocklen);
+
+	/* Track the bytes were erased */
+	memset(&g_state.was_modified[blockaddr], true, blocklen);
+	/* Clear the records of previous verification, if there were any */
+	memset(&g_state.was_verified[blockaddr], false, blocklen);
+
 	return 0;
 }
 
@@ -1229,7 +1503,7 @@ static void erase_unwritable_regions_skipflag_on_test_success(void **state)
 
 	struct flashrom_layout *layout;
 
-	setup_chip(&flashctx, &layout, param, current_test_case);
+	const chipoff_t verify_end_boundary = setup_chip(&flashctx, &layout, param, current_test_case);
 
 	// This test needs special block erase to emulate protected regions.
 	memcpy(g_test_erase_injector,
@@ -1266,6 +1540,13 @@ static void erase_unwritable_regions_skipflag_on_test_success(void **state)
 	int eraseblocks_invocations = (g_state.eraseblocks_actual_ind ==
 					current_test_case->eraseblocks_expected_ind);
 
+	int chip_verified = 1;
+	for (unsigned int i = 0; i <= verify_end_boundary; i++)
+		if (g_state.was_modified[i] && !g_state.was_verified[i]) {
+			chip_verified = 0; /* byte was modified, but not verified after */
+			printf("Error: byte 0x%x, modified: %d, verified: %d\n", i, g_state.was_modified[i], g_state.was_verified[i]);
+		}
+
 	if (chip_erased)
 		printf("Erased chip memory state for %s is CORRECT\n",
 			current_test_case->erase_test_name);
@@ -1289,10 +1570,18 @@ static void erase_unwritable_regions_skipflag_on_test_success(void **state)
 			current_test_case->eraseblocks_expected_ind,
 			g_state.eraseblocks_actual_ind);
 
+	if (chip_verified)
+		printf("Erased chip memory state for %s was verified successfully\n",
+			current_test_case->erase_test_name);
+	else
+		printf("Erased chip memory state for %s was NOT verified completely\n",
+			current_test_case->erase_test_name);
+
 	all_erase_tests_result |= ret;
 	all_erase_tests_result |= !chip_erased;
 	all_erase_tests_result |= !eraseblocks_in_order;
 	all_erase_tests_result |= !eraseblocks_invocations;
+	all_erase_tests_result |= !chip_verified;
 
 	teardown_chip(&layout);
 
@@ -1313,11 +1602,12 @@ static void write_unwritable_regions_skipflag_on_test_success(void **state) {
 	int all_write_tests_result = 0;
 	struct flashrom_flashctx flashctx = { 0 };
 	uint8_t newcontents[MIN_BUF_SIZE];
+	uint8_t newcontents_protected[MIN_BUF_SIZE];
 	const char *param = ""; /* Default values for all params. */
 
 	struct flashrom_layout *layout;
 
-	setup_chip(&flashctx, &layout, param, current_test_case);
+	const chipoff_t verify_end_boundary = setup_chip(&flashctx, &layout, param, current_test_case);
 	memcpy(&newcontents, current_test_case->written_buf, MOCK_CHIP_SIZE);
 
 	// This test needs special block erase to emulate protected regions.
@@ -1334,6 +1624,9 @@ static void write_unwritable_regions_skipflag_on_test_success(void **state) {
 
 	flashrom_flag_set(&flashctx, FLASHROM_FLAG_SKIP_UNWRITABLE_REGIONS, true);
 	flashrom_flag_set(&flashctx, FLASHROM_FLAG_SKIP_UNREADABLE_REGIONS, true);
+	flashrom_flag_set(&flashctx, FLASHROM_FLAG_VERIFY_WHOLE_CHIP, false);
+	/* We need to manually trigger verify op after write, because of protected regions */
+	flashrom_flag_set(&flashctx, FLASHROM_FLAG_VERIFY_AFTER_WRITE, false);
 
 	// We use dummyflasher programmer in tests, but for this test we need to
 	// replace dummyflasher's default get_region fn with test one.
@@ -1347,8 +1640,25 @@ static void write_unwritable_regions_skipflag_on_test_success(void **state) {
 	int ret = flashrom_image_write(&flashctx, &newcontents, MIN_BUF_SIZE, NULL);
 	printf("%s returned %d.\n", current_test_case->write_test_name, ret);
 
+	/* Expected end result leaves protected region untouched */
+	memcpy(&newcontents_protected, current_test_case->written_protected_buf, MOCK_CHIP_SIZE);
+	/* Outside of MOCK_CHIP_SIZE newcontents are not initialised in test cases, so just copy */
+	memcpy(&newcontents_protected[MOCK_CHIP_SIZE],
+		&newcontents[MOCK_CHIP_SIZE],
+		MIN_BUF_SIZE - MOCK_CHIP_SIZE);
+	printf("%s verification started.\n", current_test_case->write_test_name);
+	ret = flashrom_image_verify(&flashctx, &newcontents_protected, MIN_BUF_SIZE);
+	printf("%s verification returned %d.\n", current_test_case->write_test_name, ret);
+
 	int chip_written = !memcmp(g_state.buf, current_test_case->written_protected_buf, MOCK_CHIP_SIZE);
 
+	int chip_verified = 1;
+	for (unsigned int i = 0; i <= verify_end_boundary; i++)
+		if (g_state.was_modified[i] && !g_state.was_verified[i]) {
+			chip_verified = 0; /* byte was modified, but not verified after */
+			printf("Error: byte 0x%x, modified: %d, verified: %d\n", i, g_state.was_modified[i], g_state.was_verified[i]);
+		}
+
 	if (chip_written)
 		printf("Written chip memory state for %s is CORRECT\n",
 			current_test_case->write_test_name);
@@ -1356,8 +1666,16 @@ static void write_unwritable_regions_skipflag_on_test_success(void **state) {
 		printf("Written chip memory state for %s is WRONG\n",
 			current_test_case->write_test_name);
 
+	if (chip_verified)
+		printf("Written chip memory state for %s was verified successfully\n",
+			current_test_case->write_test_name);
+	else
+		printf("Written chip memory state for %s was NOT verified completely\n",
+			current_test_case->write_test_name);
+
 	all_write_tests_result |= ret;
 	all_write_tests_result |= !chip_written;
+	all_write_tests_result |= !chip_verified;
 
 	teardown_chip(&layout);
 
@@ -1392,3 +1710,76 @@ static void test_erase_fails_for_unwritable_region(void **state) {
 
 	assert_int_not_equal(ret, 0);
 }
+
+static void test_write_with_sacrifice_ratio50(void **state) {
+	struct test_case* current_test_case = &test_cases[20];
+
+	int all_write_test_result = 0;
+	struct flashrom_flashctx flashctx = {
+		/* If eraseblocks of smaller size fill in more than a half of the area,
+		 * erase one larger size block instead. */
+		.sacrifice_ratio = 50,
+	};
+	/* Custom expectations because sacrifice ratio is modified (not default).  */
+	struct erase_invoke eraseblocks_expected = {0x0, 0x10, TEST_ERASE_INJECTOR_5};
+	unsigned int eraseblocks_expected_ind = 1;
+
+	uint8_t newcontents[MIN_BUF_SIZE];
+	const char *param = ""; /* Default values for all params. */
+
+	struct flashrom_layout *layout;
+
+	const chipoff_t verify_end_boundary = setup_chip(&flashctx, &layout, param, current_test_case);
+	memcpy(&newcontents, current_test_case->written_buf, MOCK_CHIP_SIZE);
+
+	printf("%s started.\n", __func__);
+	int ret = flashrom_image_write(&flashctx, &newcontents, MIN_BUF_SIZE, NULL);
+	printf("%s returned %d.\n", __func__, ret);
+
+	int chip_written = !memcmp(g_state.buf, current_test_case->written_buf, MOCK_CHIP_SIZE);
+
+	int eraseblocks_in_order = !memcmp(g_state.eraseblocks_actual, &eraseblocks_expected,
+						eraseblocks_expected_ind * sizeof(struct erase_invoke));
+
+	int eraseblocks_invocations = (g_state.eraseblocks_actual_ind == eraseblocks_expected_ind);
+
+	int chip_verified = 1;
+	for (unsigned int i = 0; i <= verify_end_boundary; i++)
+		if (g_state.was_modified[i] && !g_state.was_verified[i]) {
+			chip_verified = 0; /* the byte was modified, but not verified after */
+			printf("Error: byte 0x%x, modified: %d, verified: %d\n", i, g_state.was_modified[i], g_state.was_verified[i]);
+		}
+
+	if (chip_written)
+		printf("Written chip memory state for %s is CORRECT\n", __func__);
+	else
+		printf("Written chip memory state for %s is WRONG\n", __func__);
+
+	if (eraseblocks_in_order)
+		printf("Eraseblocks order of invocation for %s is CORRECT\n", __func__);
+	else
+		printf("Eraseblocks order of invocation for %s is WRONG\n", __func__);
+
+	if (eraseblocks_invocations)
+		printf("Eraseblocks number of invocations for %s is CORRECT\n", __func__);
+	else
+		printf("Eraseblocks number of invocations for %s is WRONG, expected %d actual %d\n",
+			__func__,
+			eraseblocks_expected_ind,
+			g_state.eraseblocks_actual_ind);
+
+	if (chip_verified)
+		printf("Written chip memory state for %s was verified successfully\n", __func__);
+	else
+		printf("Written chip memory state for %s was NOT verified completely\n", __func__);
+
+	all_write_test_result |= ret;
+	all_write_test_result |= !chip_written;
+	all_write_test_result |= !eraseblocks_in_order;
+	all_write_test_result |= !eraseblocks_invocations;
+	all_write_test_result |= !chip_verified;
+
+	teardown_chip(&layout);
+
+	assert_int_equal(0, all_write_test_result);
+}
