@@ -31,8 +31,10 @@
 void run_basic_lifecycle(void **state, const struct io_mock *io,
 		const struct programmer_entry *prog, const char *param);
 
-void run_probe_lifecycle(void **state, const struct io_mock *io,
-		const struct programmer_entry *prog, const char *param, const char *const chip_name);
+void run_probe_v2_lifecycle(void **state, const struct io_mock *io,
+		const struct programmer_entry *prog, const char *param,
+		const char *const chip_name,
+		const char **expected_matched_names, unsigned int expected_matched_count);
 
 void run_init_error_path(void **state, const struct io_mock *io,
 		const struct programmer_entry *prog, const char *param, const int error_code);
